@@ -648,7 +648,7 @@ brasero_app_is_running (BraseroApp *app)
 	BraseroAppPrivate *priv;
 
 	priv = BRASERO_APP_PRIVATE (app);
-	return (priv->mainwin != NULL);
+	return priv->mainwin_running;
 }
 
 void
@@ -1149,6 +1149,7 @@ brasero_app_recent_open (GtkRecentChooser *chooser,
 		return;
 	}
 
+	/* Make sure it is no longer one shot */
 	brasero_project_manager_open_by_mime (BRASERO_PROJECT_MANAGER (priv->projects),
 					      uri,
 					      mime);
