@@ -1788,6 +1788,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	BraseroDrive *drive;
 	gchar *primary = NULL;
 	GtkWidget *make_another = NULL;
+	GtkWidget *create_cover = NULL;
 
 	drive = brasero_burn_session_get_burner (dialog->priv->session);
 	if (dialog->priv->input.type != BRASERO_TRACK_TYPE_DISC)
@@ -1866,7 +1867,7 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	|| (dialog->priv->input.type == BRASERO_TRACK_TYPE_DISC
 	&& (dialog->priv->input.subtype.media & BRASERO_MEDIUM_HAS_AUDIO))) {
 		/* since we succeed offer the possibility to create cover if that's an audio disc */
-		gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Create Cover"), GTK_RESPONSE_CLOSE);
+		create_cover = gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Create Cover"), GTK_RESPONSE_CLOSE);
 	}
 
 	res = brasero_burn_dialog_success_run (dialog);
@@ -1874,6 +1875,9 @@ brasero_burn_dialog_notify_success (BraseroBurnDialog *dialog)
 	if (make_another)
 		gtk_widget_destroy (make_another);
 
+	if (create_cover)
+		gtk_widget_destroy (create_cover);
+
 	return res;
 }
 
@@ -2032,14 +2036,13 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 	BraseroMedia media;
 	BraseroBurnResult result;
 
+	g_object_ref (session);
 	dialog->priv->session = session;
 
 	/* disable autoconfiguration */
 	if (BRASERO_IS_SESSION_CFG (dialog->priv->session))
 		brasero_session_cfg_disable (BRASERO_SESSION_CFG (dialog->priv->session));
 
-	g_object_ref (session);
-
 	/* update what we should display */
 	brasero_burn_session_get_input_type (session, &dialog->priv->input);
 	if (brasero_burn_session_is_dest_file (session))
@@ -2057,8 +2060,8 @@ brasero_burn_dialog_run (BraseroBurnDialog *dialog,
 		result = brasero_burn_dialog_record_session (dialog, media);
 	} while (result == BRASERO_BURN_RETRY);
 
-	g_object_unref (dialog->priv->session);
 	dialog->priv->session = NULL;
+	g_object_unref (session);
 
 	return (result == BRASERO_BURN_OK);
 }
