@@ -231,9 +231,9 @@ brasero_file_monitor_moved_from_event (BraseroFileMonitor *self,
 			
 	/* we remember this move for 5s. If 5s later we haven't received
 	 * a corresponding MOVED_TO then we consider the file was removed. */
-	data->id = g_timeout_add (5000,
-				  (GSourceFunc) brasero_file_monitor_move_timeout_cb,
-				  self);
+	data->id = g_timeout_add_seconds (5,
+					  (GSourceFunc) brasero_file_monitor_move_timeout_cb,
+					  self);
 
 	/* NOTE: the order is important, we _must_ append them */
 	priv->moved_list = g_slist_append (priv->moved_list, data);
@@ -473,8 +473,8 @@ brasero_file_monitor_inotify_monitor_cb (GIOChannel *channel,
 	gpointer callback_data;
 	GError *err = NULL;
 	GIOStatus status;
-	guint size;
 	gchar *name;
+	gsize size;
 
 	priv = BRASERO_FILE_MONITOR_PRIVATE (self);
 	while (condition & G_IO_IN) {
@@ -544,12 +544,12 @@ brasero_file_monitor_inotify_monitor_cb (GIOChannel *channel,
 								      &event);
 			}
 			else
-				g_warning ("Unknown (or already deleted) monitored directory = > ignored \n");
+				g_warning ("Unknown (or already deleted) monitored directory = > ignored");
 		}
 		else {
 			GSList *list;
 
-			/* This is an even happening on the top directory there*/
+			/* This is an event happening on the top directory there*/
 			list = callback_data;
 			brasero_file_monitor_inotify_file_event (self,
 								 list,
@@ -703,7 +703,12 @@ brasero_file_monitor_foreach_cancel_file_cb (gpointer key,
 			result->key = key;
 			result->callback_data = file_data;
 			data->results = g_slist_prepend (data->results, result);
-			break;
+
+			/* NOTE: don't stop here as func (at least for data 
+			 * projects returns TRUE when:
+			 * - callback_data is the data looked for
+			 * - when there it is an ancestor
+			 * So it's never finished with just one hit. */
 		}
 	}
 }
@@ -739,7 +744,7 @@ brasero_file_monitor_foreach_cancel (BraseroFileMonitor *self,
 	data.func = func;
 	data.results = NULL;
 	data.callback_data = callback_data;
-	data.dev_fd = g_io_channel_unix_get_fd (priv->notify);;
+	data.dev_fd = g_io_channel_unix_get_fd (priv->notify);
 
 	g_hash_table_foreach (priv->files,
 			      brasero_file_monitor_foreach_cancel_file_cb,
