@@ -39,8 +39,8 @@
 #include "brasero-tool-dialog.h"
 #include "burn-session.h"
 #include "burn.h"
-#include "burn-medium.h"
-#include "burn-drive.h"
+#include "brasero-medium.h"
+#include "brasero-drive.h"
 
 G_DEFINE_TYPE (BraseroToolDialog, brasero_tool_dialog, GTK_TYPE_DIALOG);
 
@@ -301,8 +301,8 @@ void
 brasero_tool_dialog_set_medium_type_shown (BraseroToolDialog *self,
 					   BraseroMediaType media_type)
 {
-	brasero_medium_selection_show_type (BRASERO_MEDIUM_SELECTION (self->priv->selector),
-					    media_type);
+	brasero_medium_selection_show_media_type (BRASERO_MEDIUM_SELECTION (self->priv->selector),
+						  media_type);
 }
 
 BraseroMedium *
@@ -322,20 +322,15 @@ brasero_tool_dialog_set_medium (BraseroToolDialog *self,
 }
 
 static void
-brasero_tool_dialog_drive_changed_cb (GtkComboBox *combo_box,
+brasero_tool_dialog_drive_changed_cb (BraseroMediumSelection *selection,
+				      BraseroMedium *medium,
 				      BraseroToolDialog *self)
 {
 	BraseroToolDialogClass *klass;
-	BraseroMedium *medium;
-
-	medium = brasero_medium_selection_get_active (BRASERO_MEDIUM_SELECTION (combo_box));
 
 	klass = BRASERO_TOOL_DIALOG_GET_CLASS (self);
 	if (klass->drive_changed)
 		klass->drive_changed (self, medium);
-
-	if (medium)
-		g_object_unref (medium);
 }
 
 static gboolean
@@ -383,7 +378,6 @@ brasero_tool_dialog_cancel_dialog (GtkWidget *toplevel)
 gboolean
 brasero_tool_dialog_cancel (BraseroToolDialog *self)
 {
-	BraseroBurnResult result = BRASERO_BURN_OK;
 	BraseroToolDialogClass *klass;
 
 	klass = BRASERO_TOOL_DIALOG_GET_CLASS (self);
@@ -481,10 +475,11 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 			    FALSE, FALSE, 0);
 	g_free (title_str);
 
-	brasero_medium_selection_show_type (BRASERO_MEDIUM_SELECTION (obj->priv->selector),
-					    BRASERO_MEDIA_TYPE_REWRITABLE|
-					    BRASERO_MEDIA_TYPE_WRITABLE|
-					    BRASERO_MEDIA_TYPE_READABLE);
+	brasero_medium_selection_show_media_type (BRASERO_MEDIUM_SELECTION (obj->priv->selector),
+						  BRASERO_MEDIA_TYPE_REWRITABLE|
+						  BRASERO_MEDIA_TYPE_WRITABLE|
+						  BRASERO_MEDIA_TYPE_AUDIO|
+						  BRASERO_MEDIA_TYPE_DATA);
 
 	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (obj)->vbox),
 			    obj->priv->upper_box,
@@ -538,7 +533,7 @@ brasero_tool_dialog_init (BraseroToolDialog *obj)
 				      GTK_RESPONSE_CANCEL);
 
 	g_signal_connect (G_OBJECT (obj->priv->selector),
-			  "changed",
+			  "medium-changed",
 			  G_CALLBACK (brasero_tool_dialog_drive_changed_cb),
 			  obj);
 
