@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -392,6 +394,8 @@ brasero_task_set_track_output_size_default (BraseroTask *self,
 
 	brasero_task_ctx_get_current_track (BRASERO_TASK_CTX (self), &track);
 	brasero_track_get_type (track, &input);
+	BRASERO_BURN_LOG_TYPE (&input, "Track type");
+
 	if (input.type == BRASERO_TRACK_TYPE_IMAGE) {
 		BraseroBurnResult result;
 		gint64 sectors = 0;
@@ -406,7 +410,6 @@ brasero_task_set_track_output_size_default (BraseroTask *self,
 			return result;
 
 		BRASERO_BURN_LOG ("Got a default image track length %lli", sectors);
-
 		brasero_task_ctx_set_output_size_for_current_track (BRASERO_TASK_CTX (self),
 								    sectors,
 								    size);
@@ -482,8 +485,7 @@ brasero_task_start_items (BraseroTask *self,
 			/* "fake mode" to get size. Forgive the jobs that cannot
 			 * retrieve the size for one track. Just deactivate and
 			 * go on with the next.
-			 * NOTE: after this result the job is no longer active
-			 */
+			 * NOTE: after this result the job is no longer active */
 			continue;
 		}
 
@@ -541,7 +543,7 @@ brasero_task_start (BraseroTask *self,
 	/* Activate all items that can be. If no item can be then skip */
 	result = brasero_task_activate_items (self, error);
 	if (result == BRASERO_BURN_NOT_RUNNING) {
-		BRASERO_BURN_LOG ("task skipped");
+		BRASERO_BURN_LOG ("Task skipped");
 		return BRASERO_BURN_OK;
 	}
 
