@@ -7,19 +7,21 @@
  ***************************************************************************/
 
 /*
- *  This program is free software; you can redistribute it and/or modify
+ *  Brasero is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
  *  the Free Software Foundation; either version 2 of the License, or
  *  (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
+ *  Brasero is distributed in the hope that it will be useful,
  *  but WITHOUT ANY WARRANTY; without even the implied warranty of
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU Library General Public License for more details.
  *
  *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *  along with this program; if not, write to:
+ * 	The Free Software Foundation, Inc.,
+ * 	51 Franklin Street, Fifth Floor
+ * 	Boston, MA  02110-1301, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -76,89 +78,46 @@ static GObjectClass *parent_class = NULL;
 static BraseroBurnResult
 brasero_wodim_stderr_read (BraseroProcess *process, const gchar *line)
 {
-	BraseroWodim *wodim = BRASERO_WODIM (process);
-	BraseroWodimPrivate *priv;
 	BraseroBurnFlag flags;
 
-	priv = BRASERO_WODIM_PRIVATE (wodim);
-	brasero_job_get_flags (BRASERO_JOB (wodim), &flags);
+	brasero_job_get_flags (BRASERO_JOB (process), &flags);
 
-	if (strstr (line, "Cannot open SCSI driver.")) {
+	if (strstr (line, "Cannot open SCSI driver.")
+	||  strstr (line, "Operation not permitted. Cannot send SCSI cmd via ioctl")
+	||  strstr (line, "Cannot open or use SCSI driver")) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_PERMISSION,
-						_("You don't seem to have the required permission to use this drive")));
-	}
-	else if (strstr (line, "No disk / Wrong disk!") != NULL) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_MEDIA_NONE,
-						_("There doesn't seem to be a disc in the drive")));
-	}
-	else if (strstr (line, "Input buffer error, aborting") != NULL) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("input buffer error")));
-	}
-	else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
-		/* NOTE: defer the consequence of this error as it is not always
-		 * fatal. So send a warning but don't stop the process. */
-		brasero_process_deferred_error (process,
-						g_error_new (BRASERO_BURN_ERROR,
-							     BRASERO_BURN_ERROR_MEDIA_NOT_WRITABLE,
-							     _("The CD has already been recorded")));
-	}
-	else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_MEDIA_NOT_REWRITABLE,
-						_("The CD cannot be blanked")));
+						_("You do not have the required permissions to use this drive")));
 	}
 	else if (!(flags & BRASERO_BURN_FLAG_OVERBURN)
 	     &&  strstr (line, "Data may not fit on current disk")) {
 		/* we don't error out if overburn was chosen */
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_MEDIA_SPACE,
-						_("The files selected did not fit on the CD")));
+						BRASERO_BURN_ERROR_MEDIUM_SPACE,
+						_("Not enough space available on the disc")));
 	}
-	else if (strstr (line ,"wodim: A write error occured")) {
+	else if (strstr (line ,"cdrecord: A write error occured")
+	     ||  strstr (line, "Could not write Lead-in")
+	     ||  strstr (line, "Cannot fixate disk")) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("a write error occured which was likely due to overburning the disc")));
-	}
-	else if (strstr (line, "Inappropriate audio coding")) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_INCOMPATIBLE_FORMAT,
-						_("All audio files must be stereo, 16-bit digital audio with 44100Hz samples")));
-	}
-	else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_INCOMPATIBLE_FORMAT,
-						_("The image does not seem to be a proper iso9660 file system")));
+						BRASERO_BURN_ERROR_WRITE_MEDIUM,
+						_("An error occured while writing to disc")));
 	}
 	else if (strstr (line, "DMA speed too slow") != NULL) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_SLOW_DMA,
-						_("The system is too slow to write the CD at this speed. Try a lower speed")));
-	}
-	else if (strstr (line, "Operation not permitted. Cannot send SCSI cmd via ioctl")) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_SCSI_IOCTL,
-						_("You don't seem to have the required permission to use this drive")));
+						_("The system is too slow to write the disc at this speed. Try a lower speed")));
 	}
 	else if (strstr (line, "Device or resource busy")) {
 		if (!strstr (line, "retrying in")) {
 			brasero_job_error (BRASERO_JOB (process),
 					   g_error_new (BRASERO_BURN_ERROR,
-							BRASERO_BURN_ERROR_BUSY_DRIVE,
-							_("The drive seems to be busy (maybe check you have proper permissions to use it)")));
+							BRASERO_BURN_ERROR_DRIVE_BUSY,
+							_("The drive is busy")));
 		}
 	}
 	else if (strstr (line, "Illegal write mode for this drive")) {
@@ -166,40 +125,77 @@ brasero_wodim_stderr_read (BraseroProcess *process, const gchar *line)
 		 * drive with cdrdao and eject it. Should we ? */
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_BUSY_DRIVE,
-						_("The drive seems to be busy (maybe you should reload the media)")));
+						BRASERO_BURN_ERROR_DRIVE_BUSY,
+						_("The drive is busy")));
 	}
-	else if (strstr (line, "wodim: No such file or directory. Cannot open")) {
+	else if (strstr (line, "Probably trying to use ultra high speed+ medium on improper writer")) {
+		/* Set a deferred error as this message tends to indicate a failure */
+		brasero_process_deferred_error (process,
+						g_error_new (BRASERO_BURN_ERROR,
+							     BRASERO_BURN_ERROR_MEDIUM_INVALID,
+							     _("The disc is not supported")));
+	}
+	/* REMINDER: these should not be necessary as we checked that already */
+	/**
+	else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("the image file cannot be found")));
+						BRASERO_BURN_ERROR_INPUT_INVALID,
+						_("The image does not seem to be a proper iso9660 file system")));
 	}
-	else if (strstr (line, "Bad file descriptor. read error on input file")
-	     ||  strstr (line, "No tracks specified. Need at least one.")) {
+	else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
+	**/	/* NOTE: defer the consequence of this error as it is not always
+		 * fatal. So send a warning but don't stop the process. */
+	/**	brasero_process_deferred_error (process,
+						g_error_new (BRASERO_BURN_ERROR,
+							     BRASERO_BURN_ERROR_MEDIUM_INVALID,
+							     _("The disc is already burnt")));
+	}
+	else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("internal error")));
-	}
-	else if (strstr (line, "Could not write Lead-in")) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("the cd information could not be written")));
-	}
-	else if (strstr (line, "Cannot fixate disk")) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("the disc could not be closed")));
+						BRASERO_BURN_ERROR_MEDIUM_INVALID,
+						_("The disc could not be blanked")));
 	}
 	else if (strstr (line, "Bad audio track size")) {
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
 						BRASERO_BURN_ERROR_GENERAL,
-						_("the audio tracks are too short or not a multiple of 2352")));
+						_("The audio tracks are too short or not a multiple of 2352")));
 	}
+	else if (strstr (line, "cdrecord: No such file or directory. Cannot open")
+	     ||  strstr (line, "No tracks specified. Need at least one.")) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_INPUT,
+						_("The image file cannot be found")));
+	}
+	else if (strstr (line, "Inappropriate audio coding")) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_INPUT_INVALID,
+						_("All audio files must be stereo, 16-bit digital audio with 44100Hz samples")));
+	}
+	else if (strstr (line, "No disk / Wrong disk!") != NULL) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_MEDIA_NONE,
+						_("There seems to be no disc in the drive")));
+	}
+
+	**/
+
+	/** For these we'd rather have a message saying "cdrecord failed"
+	 *  as an internal error occured says nothing/even less
+	else if (strstr (line, "Bad file descriptor. read error on input file")
+	     ||  strstr (line, "Input buffer error, aborting")) {
+		brasero_job_error (BRASERO_JOB (process),
+				   g_error_new (BRASERO_BURN_ERROR,
+						BRASERO_BURN_ERROR_GENERAL,
+						_("An internal error occured")));
+	}
+
+	**/
 
 	return BRASERO_BURN_OK;
 }
@@ -212,7 +208,7 @@ brasero_wodim_compute (BraseroWodim *wodim,
 {
 	gboolean track_num_changed = FALSE;
 	BraseroWodimPrivate *priv;
-	gchar *action_string;
+	BraseroJobAction action;
 	gint64 this_remain;
 	gint64 bytes;
 	gint64 total;
@@ -233,12 +229,24 @@ brasero_wodim_compute (BraseroWodim *wodim,
 	bytes = (total - priv->current_track_end_pos) + this_remain;
 	brasero_job_set_written_session (BRASERO_JOB (wodim), total - bytes);
 
-	action_string = g_strdup_printf ("Writing track %02i", track_num);
-	brasero_job_set_current_action (BRASERO_JOB (wodim),
-					BRASERO_BURN_ACTION_RECORDING,
-					action_string,
-					track_num_changed);
-	g_free (action_string);
+	brasero_job_get_action (BRASERO_JOB (wodim), &action);
+	if (action == BRASERO_JOB_ACTION_RECORD) {
+		gchar *action_string;
+
+		action_string = g_strdup_printf ("Writing track %02i", track_num);
+		brasero_job_set_current_action (BRASERO_JOB (wodim),
+						BRASERO_BURN_ACTION_RECORDING,
+						action_string,
+						track_num_changed);
+		g_free (action_string);
+	}
+	else if (action == BRASERO_JOB_ACTION_ERASE) {
+		brasero_job_set_progress (BRASERO_JOB (wodim), (gfloat) mb_written / (gfloat) mb_total);
+		brasero_job_set_current_action (BRASERO_JOB (wodim),
+						BRASERO_BURN_ACTION_BLANKING,
+						NULL,
+						FALSE);
+	}
 }
 
 static BraseroBurnResult
@@ -264,9 +272,9 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 
 		priv->current_track_written = mb_written * 1048576;
 		brasero_wodim_compute (wodim,
-					  mb_written,
-					  mb_total,
-					  track);
+				       mb_written,
+				       mb_total,
+				       track);
 
 		brasero_job_start_progress (BRASERO_JOB (wodim), FALSE);
 	} 
@@ -298,9 +306,14 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 		brasero_job_start_progress (BRASERO_JOB (wodim), FALSE);
 	}
 	else if (sscanf (line, "Formating in progress: %d.%d %% done", &mb_written, &mb_total) == 2) {
+		brasero_job_set_current_action (BRASERO_JOB (process),
+						BRASERO_BURN_ACTION_BLANKING,
+						_("Formatting disc"),
+						FALSE);
+
 		brasero_job_start_progress (BRASERO_JOB (wodim), FALSE);
 		brasero_job_set_progress (BRASERO_JOB (wodim),
-					  (gdouble) ((gdouble) mb_written + ((gdouble) mb_total) / 10.0));
+					  (gdouble) ((gdouble) mb_written + ((gdouble) mb_total) / 10.0) / 100.0);
 	}
 	else if (sscanf (line, "Track %*d: %*s %d MB ", &mb_total) == 1) {
 /*		if (mb_total > 0)
@@ -334,8 +347,8 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 
 		brasero_job_error (BRASERO_JOB (process),
 				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_RELOAD_MEDIA,
-						_("The media needs to be reloaded before being recorded")));
+						BRASERO_BURN_ERROR_MEDIUM_NEED_RELOADING,
+						_("The disc needs to be reloaded before being recorded")));
 	}
 	else if (g_str_has_prefix (line, "Fixating...")
 	     ||  g_str_has_prefix (line, "Writing Leadout...")) {
@@ -347,16 +360,20 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 	else if (g_str_has_prefix (line, "Last chance to quit, ")) {
 		brasero_job_set_dangerous (BRASERO_JOB (process), TRUE);
 	}
-	else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")
+/*	else if (g_str_has_prefix (line, "Blanking PMA, TOC, pregap")
 	     ||  strstr (line, "Blanking entire disk")) {
 
 	}
-	else if (strstr (line, "Use tsize= option in SAO mode to specify track size")) {
-		brasero_job_error (BRASERO_JOB (process),
-				   g_error_new (BRASERO_BURN_ERROR,
-						BRASERO_BURN_ERROR_GENERAL,
-						_("internal error")));
+ */
+	else if (strstr (line, "Disk sub type: Ultra High speed+")) {
+		/* Set a deferred error as this message tends to indicate a failure */
+		brasero_process_deferred_error (process,
+						g_error_new (BRASERO_BURN_ERROR,
+							     BRASERO_BURN_ERROR_MEDIUM_INVALID,
+							     _("The disc is not supported")));
 	}
+	/* This should not happen */
+	/* else if (strstr (line, "Use tsize= option in SAO mode to specify track size")) */
 
 	return BRASERO_BURN_OK;
 }
@@ -382,6 +399,7 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 	gchar buffer [128];
 	BraseroSongInfo *info;
 	BraseroWodimPrivate *priv;
+        int errsv;
 
 	priv = BRASERO_WODIM_PRIVATE (wodim);
 
@@ -424,6 +442,16 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 
 	BRASERO_JOB_LOG (wodim, "writing inf (%s)", path);
 
+	/* The problem here is that when writing CD-TEXT from .inf files, wodim
+	 * uses only one charset (and don't let us specify which one) which is
+	 * ISO-8859-1. (NOTE: don't believe the doc claiming it requires ASCII
+	 * and see cdrecord/cdtext.c line 309).
+	 * So we have to convert our UTF-8 input into such a charset.
+	 * NOTE: according to docs ASCII should be used for text packs other
+	 * than disc/track title.
+	 * It might be good in the end to write and pack CD-TEXT pack data 
+	 * ourselves so we can set a different charset from English like 
+	 * Chinese for example. */
 	info = brasero_track_get_audio_info (track);
 
 	strcpy (buffer, "# created by brasero\n");
@@ -454,8 +482,20 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 	if (b_written != size)
 		goto error;
 
-	if (album)
-		string = g_strdup_printf ("Albumtitle=\t%s\n", album);
+	if (album) {
+		gchar *encoded;
+
+		encoded = g_convert_with_fallback (album,
+						   -1,
+						   "ISO-8859-1",
+						   "UTF-8",
+						   "_",	/* Fallback for non convertible characters */
+						   NULL,
+						   NULL,
+						   NULL);
+		string = g_strdup_printf ("Albumtitle=\t%s\n", encoded);
+		g_free (encoded);
+	}
 	else
 		string = strdup ("Albumtitle=\t\n");
 	size = strlen (string);
@@ -464,8 +504,20 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 	if (b_written != size)
 		goto error;
 
-	if (info->artist)
-		string = g_strdup_printf ("Performer=\t%s\n", info->artist);
+	if (info->artist) {
+		gchar *encoded;
+
+		encoded = g_convert_with_fallback (info->artist,
+						   -1,
+						   "ISO-8859-1",
+						   "UTF-8",
+						   "_",	/* Fallback for non convertible characters */
+						   NULL,
+						   NULL,
+						   NULL);
+		string = g_strdup_printf ("Performer=\t%s\n", encoded);
+		g_free (encoded);
+	}
 	else
 		string = strdup ("Performer=\t\n");
 	size = strlen (string);
@@ -474,8 +526,20 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 	if (b_written != size)
 		goto error;
 
-	if (info->composer)
-		string = g_strdup_printf ("Composer=\t%s\n", info->composer);
+	if (info->composer) {
+		gchar *encoded;
+
+		encoded = g_convert_with_fallback (info->composer,
+						   -1,
+						   "ISO-8859-1",
+						   "UTF-8",
+						   "_",	/* Fallback for non convertible characters */
+						   NULL,
+						   NULL,
+						   NULL);
+		string = g_strdup_printf ("Composer=\t%s\n", encoded);
+		g_free (encoded);
+	}
 	else
 		string = strdup ("Composer=\t\n");
 	size = strlen (string);
@@ -484,8 +548,20 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 	if (b_written != size)
 		goto error;
 
-	if (info->title)
-		string = g_strdup_printf ("Tracktitle=\t%s\n", info->title);
+	if (info->title) {
+		gchar *encoded;
+
+		encoded = g_convert_with_fallback (info->title,
+						   -1,
+						   "ISO-8859-1",
+						   "UTF-8",
+						   "_",	/* Fallback for non convertible characters */
+						   NULL,
+						   NULL,
+						   NULL);
+		string = g_strdup_printf ("Tracktitle=\t%s\n", encoded);
+		g_free (encoded);
+	}
 	else
 		string = strdup ("Tracktitle=\t\n");
 	size = strlen (string);
@@ -561,6 +637,8 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 
 	size = strlen (string);
 	b_written = write (fd, string, size);
+	g_free (string);
+
 	if (b_written != size)
 		goto error;
 
@@ -575,14 +653,19 @@ brasero_wodim_write_inf (BraseroWodim *wodim,
 
 
 error:
+        errsv = errno;
+
 	g_remove (path);
 	g_free (path);
 
+	BRASERO_JOB_LOG (wodim,
+			 "The inf file can't be written : %s",
+			 g_strerror (errsv));
+
 	g_set_error (error,
 		     BRASERO_BURN_ERROR,
 		     BRASERO_BURN_ERROR_GENERAL,
-		     _("the inf file can't be written : %s"), 
-		     strerror (errno));
+		     _("An internal error occured"));
 
 	return BRASERO_BURN_ERR;
 }
@@ -632,6 +715,7 @@ brasero_wodim_write_infs (BraseroWodim *wodim,
 		start += BRASERO_DURATION_TO_SECTORS (length);
 	}
 
+	g_free (album);
 	g_free (tmpdir);
 
 	return BRASERO_BURN_OK;
@@ -669,6 +753,17 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 	if (flags & BRASERO_BURN_FLAG_MULTI)
 		g_ptr_array_add (argv, g_strdup ("-multi"));
 
+	/* NOTE: This write mode is necessary for all CLONE images burning */
+	if (flags & BRASERO_BURN_FLAG_RAW)
+		g_ptr_array_add (argv, g_strdup ("-raw96r"));
+
+	/* NOTE1: DAO can't be used if we're appending to a disc */
+	/* NOTE2: CD-text cannot be written in tao mode (which is the default)
+	 * NOTE3: when we don't want wodim to use stdin then we give the audio
+	 * file on the command line. Otherwise we use the .inf */
+	if (flags & BRASERO_BURN_FLAG_DAO)
+		g_ptr_array_add (argv, g_strdup ("-dao"));
+
 	brasero_job_get_input_type (BRASERO_JOB (wodim), &type);
 	if (brasero_job_get_fd_in (BRASERO_JOB (wodim), NULL) == BRASERO_BURN_OK) {
 		BraseroBurnResult result;
@@ -678,10 +773,11 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 		/* we need to know what is the type of the track (audio / data) */
 		result = brasero_job_get_input_type (BRASERO_JOB (wodim), &type);
 		if (result != BRASERO_BURN_OK) {
+			BRASERO_JOB_LOG (wodim, "Imager doesn't seem to be ready");
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("imager doesn't seem to be ready"));
+				     _("An internal error occured"));
 			return BRASERO_BURN_ERR;
 		}
 		
@@ -690,10 +786,11 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 							      &sectors,
 							      NULL);
 		if (result != BRASERO_BURN_OK) {
+			BRASERO_JOB_LOG (wodim, "The size of the session cannot be retrieved");
 			g_set_error (error,
 				     BRASERO_BURN_ERROR,
 				     BRASERO_BURN_ERROR_GENERAL,
-				     _("the size of the session cannot be retrieved"));
+				     _("An internal error occured"));
 			return BRASERO_BURN_ERR;
 		}
 
@@ -710,11 +807,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 			if (type.subtype.img_format == BRASERO_IMAGE_FORMAT_BIN) {
 				g_ptr_array_add (argv, g_strdup_printf ("tsize=%Lis", sectors));
 
-				/* DAO can't be used if we're appending to a 
-				 * disc with audio track(s) on it */
-				if (flags & BRASERO_BURN_FLAG_DAO)
-					g_ptr_array_add (argv, g_strdup ("-dao"));
-
 				g_ptr_array_add (argv, g_strdup ("-data"));
 				g_ptr_array_add (argv, g_strdup ("-nopad"));
 				g_ptr_array_add (argv, g_strdup ("-"));
@@ -723,10 +815,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 				BRASERO_JOB_NOT_SUPPORTED (wodim);
 		}
 		else if (type.type == BRASERO_TRACK_TYPE_AUDIO) {
-			/* now set the rest of the arguments */
-			if (flags & BRASERO_BURN_FLAG_DAO)
-				g_ptr_array_add (argv, g_strdup ("-dao"));
-
 			/* NOTE: when we don't want wodim to use stdin then we
 			 * give the audio file on the command line. Otherwise we
 			 * use the .inf */
@@ -748,12 +836,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 		BraseroBurnResult result;
 		GSList *tracks;
 
-		/* CD-text cannot be written in tao mode (which is the default)
-		 * NOTE: when we don't want wodim to use stdin then we give the
-		 * audio file on the command line. Otherwise we use the .inf */
-		if (flags & BRASERO_BURN_FLAG_DAO)
-			g_ptr_array_add (argv, g_strdup ("-dao"));
-
 		g_ptr_array_add (argv, g_strdup ("fs=16m"));
 		g_ptr_array_add (argv, g_strdup ("-audio"));
 		g_ptr_array_add (argv, g_strdup ("-swab"));
@@ -793,9 +875,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 			if (!image_path)
 				BRASERO_JOB_NOT_READY (wodim);
 
-			if (flags & BRASERO_BURN_FLAG_DAO)
-				g_ptr_array_add (argv, g_strdup ("-dao"));
-
 			g_ptr_array_add (argv, g_strdup ("fs=16m"));
 			g_ptr_array_add (argv, g_strdup ("-data"));
 			g_ptr_array_add (argv, g_strdup ("-nopad"));
@@ -808,9 +887,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 			if (!isopath)
 				BRASERO_JOB_NOT_READY (wodim);
 
-			if (flags & BRASERO_BURN_FLAG_DAO)
-				g_ptr_array_add (argv, g_strdup ("-dao"));
-
 			g_ptr_array_add (argv, g_strdup ("fs=16m"));
 			g_ptr_array_add (argv, g_strdup ("-data"));
 			g_ptr_array_add (argv, g_strdup ("-nopad"));
@@ -823,13 +899,7 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 			if (!rawpath)
 				BRASERO_JOB_NOT_READY (wodim);
 
-			/* NOTE: we ignore DAO flag on purpose since it isn't
-			 * implemented yet. Don't error out since there is no
-			 * way for us to tell that we don't support this flag
-			 * for this specific input. */
-
 			g_ptr_array_add (argv, g_strdup ("fs=16m"));
-			g_ptr_array_add (argv, g_strdup ("-raw96r"));
 			g_ptr_array_add (argv, g_strdup ("-clone"));
 			g_ptr_array_add (argv, rawpath);
 		}
@@ -841,9 +911,6 @@ brasero_wodim_set_argv_record (BraseroWodim *wodim,
 			if (!cuepath)
 				BRASERO_JOB_NOT_READY (wodim);
 
-			if (flags & BRASERO_BURN_FLAG_DAO)
-				g_ptr_array_add (argv, g_strdup ("-dao"));
-
 			g_ptr_array_add (argv, g_strdup ("fs=16m"));
 
 			cue_str = g_strdup_printf ("cuefile=%s", cuepath);
@@ -883,9 +950,60 @@ brasero_wodim_set_argv_blank (BraseroWodim *wodim, GPtrArray *argv)
 					    (flags & BRASERO_BURN_FLAG_FAST_BLANK) ? "fast" : "all");
 		g_ptr_array_add (argv, blank_str);
 	}
-	else
+	else if (media & BRASERO_MEDIUM_UNFORMATTED) {
 		g_ptr_array_add (argv, g_strdup ("-format"));
 
+		/* There are many options for this given by an option that is
+		 * not documented: formattype=#type where type can be:
+		 * - force: to force reformatting => problem cannot/doesn't want to reformat a disc already formated
+		 * - full: to do a full formatting => problem only reformats what is not done yet
+		 * - background: formatting in the background => problem cannot/doesn't want to reformat a disc already formated
+		 * conclusion: we can only accept unformated media.
+		 * The following option allows to do it more quickly:
+		 * g_ptr_array_add (argv, g_strdup ("formattype=background");
+		 */
+				
+	}
+	else {
+		guint speed;
+		BraseroWodimPrivate *priv;
+
+		priv = BRASERO_WODIM_PRIVATE (wodim);
+
+		/* Since we can't reformat any already formatted DVD+RW, we 
+		 * write 0s to it to blank. */
+		if (priv->immediate) {
+			g_ptr_array_add (argv, g_strdup ("-immed"));
+			g_ptr_array_add (argv, g_strdup_printf ("minbuf=%i", priv->minbuf));
+		}
+
+		if (brasero_job_get_speed (BRASERO_JOB (wodim), &speed) == BRASERO_BURN_OK) {
+			gchar *speed_str;
+
+			speed_str = g_strdup_printf ("speed=%d", speed);
+			g_ptr_array_add (argv, speed_str);
+		}
+
+		if (!(flags & BRASERO_BURN_FLAG_FAST_BLANK)) {
+			gint64 sectors = 0;
+			BraseroMedium *medium;
+
+			brasero_job_get_medium (BRASERO_JOB (wodim), &medium);
+			brasero_medium_get_data_size (medium, NULL, &sectors);
+			if (!sectors)
+				brasero_medium_get_capacity (medium, NULL, &sectors);
+
+			g_ptr_array_add (argv, g_strdup_printf ("tsize=%Lis", sectors));
+		}
+		else	/* we set 512s because wodim complains otherwise */
+			g_ptr_array_add (argv, g_strdup_printf ("tsize=512s"));
+
+		g_ptr_array_add (argv, g_strdup ("fs=16m"));
+		g_ptr_array_add (argv, g_strdup ("-data"));
+		g_ptr_array_add (argv, g_strdup ("-nopad"));
+		g_ptr_array_add (argv, g_strdup ("/dev/zero"));
+	}
+
 	brasero_job_set_current_action (BRASERO_JOB (wodim),
 					BRASERO_BURN_ACTION_BLANKING,
 					NULL,
@@ -925,8 +1043,12 @@ brasero_wodim_set_argv (BraseroProcess *process,
         if (flags & BRASERO_BURN_FLAG_DUMMY)
 		g_ptr_array_add (argv, g_strdup ("-dummy"));
 
-	if (flags & BRASERO_BURN_FLAG_NOGRACE)
-		g_ptr_array_add (argv, g_strdup ("gracetime=0"));
+	/* There is a bug in wodim where if we set that it takes 15 more so 
+	 * disable it as long as the bug remains. */
+
+	/* if (flags & BRASERO_BURN_FLAG_NOGRACE)
+	 *	g_ptr_array_add (argv, g_strdup ("gracetime=0"));
+	 */
 
 	if (action == BRASERO_JOB_ACTION_RECORD)
 		result = brasero_wodim_set_argv_record (wodim, argv, error);
@@ -1055,7 +1177,7 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 	/* NOTE: it seems that cdrecord can burn cue files on the fly */
 	brasero_plugin_define (plugin,
 			       "wodim",
-			       _("use wodim to burn CDs"),
+			       _("Use wodim to burn CDs and DVDs"),
 			       "Philippe Rouquier",
 			       0);
 
@@ -1069,35 +1191,68 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 					BRASERO_PLUGIN_IO_ACCEPT_FILE,
 					BRASERO_IMAGE_FORMAT_BIN);
 
-	/* wodim can burn all DVDs when it's ISOs */
+	/* wodim can burn all DVDs (except DVD-RW restricted) when it's ISOs */
 	output = brasero_caps_disc_new (dvd_media);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 
+	/* All CD-R(W) */
 	output = brasero_caps_disc_new (media);
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (input);
 
-	input = brasero_caps_image_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
-					BRASERO_IMAGE_FORMAT_CUE|
-					BRASERO_IMAGE_FORMAT_CLONE);
+	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
+					BRASERO_PLUGIN_IO_ACCEPT_FILE,
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100|
+					BRASERO_METADATA_INFO);
 
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (input);
 
 	input = brasero_caps_audio_new (BRASERO_PLUGIN_IO_ACCEPT_PIPE|
 					BRASERO_PLUGIN_IO_ACCEPT_FILE,
-					BRASERO_AUDIO_FORMAT_RAW);
+					BRASERO_AUDIO_FORMAT_RAW|
+					BRASERO_AUDIO_FORMAT_44100);
 
 	brasero_plugin_link_caps (plugin, output, input);
 	g_slist_free (output);
 	g_slist_free (input);
 
+	/* for CLONE and CUE type images, we only want blank CD-R(W) */
+	output = brasero_caps_disc_new (BRASERO_MEDIUM_CD|
+					BRASERO_MEDIUM_WRITABLE|
+					BRASERO_MEDIUM_REWRITABLE|
+					BRASERO_MEDIUM_BLANK);
+
+	input = brasero_caps_image_new (BRASERO_PLUGIN_IO_ACCEPT_FILE,
+					BRASERO_IMAGE_FORMAT_CUE|
+					BRASERO_IMAGE_FORMAT_CLONE);
+
+	brasero_plugin_link_caps (plugin, output, input);
+	g_slist_free (output);
+	g_slist_free (input);
+
+	/* Flags for CD (RW)s */
+	BRASERO_PLUGIN_ADD_STANDARD_CDR_FLAGS (plugin, BRASERO_BURN_FLAG_NONE);
+	BRASERO_PLUGIN_ADD_STANDARD_CDRW_FLAGS (plugin, BRASERO_BURN_FLAG_NONE);
+
+	/* Apart from DAO it also supports RAW mode to burn CLONE images. This
+	 * is a special mode for which there isn't any DUMMY burn possible */
+	brasero_plugin_set_flags (plugin,
+				  BRASERO_MEDIUM_CD|
+				  BRASERO_MEDIUM_WRITABLE|
+				  BRASERO_MEDIUM_REWRITABLE|
+				  BRASERO_MEDIUM_BLANK,
+				  BRASERO_BURN_FLAG_RAW|
+				  BRASERO_BURN_FLAG_BURNPROOF|
+				  BRASERO_BURN_FLAG_OVERBURN|
+				  BRASERO_BURN_FLAG_NOGRACE,
+				  BRASERO_BURN_FLAG_NONE);
+
 	/* For DVD-W and DVD-RW sequential
 	 * NOTE: given the performed tests it seems that wodim should not be 
-	 * used to start a multisession DVD-RW or even continue one.
-	 * I didn't try with DVD-W since I didn't want to waste one for nothing.
-	 */
+	 * used to start a multisession DVD-RW or even continue one. */
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_DVD|
 				  BRASERO_MEDIUM_SEQUENTIAL|
@@ -1111,6 +1266,7 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
 
+	/* For DVD+R limited capabilites to blank media */
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_DVDR_PLUS|
 				  BRASERO_MEDIUM_BLANK,
@@ -1120,13 +1276,11 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
 
-	/* for DVD+RW 
-	 * NOTE: we don't accept unformatted media here */
+	/* For DVD+RW: limited capabilities there are no MULTI possible
+	 * NOTE: no UNFORMATTED here since wodim doesn't format them before*/
 	brasero_plugin_set_flags (plugin,
 				  BRASERO_MEDIUM_DVDRW_PLUS|
 				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
@@ -1136,40 +1290,7 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 				  BRASERO_MEDIUM_APPENDABLE|
 				  BRASERO_MEDIUM_CLOSED|
 				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_BURNPROOF|
 				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_NOGRACE|
-				  BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	/* Flags for CD (RW)s */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_CD|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_BLANK,
-				  BRASERO_BURN_FLAG_DAO|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
-				  BRASERO_BURN_FLAG_NOGRACE,
-				  BRASERO_BURN_FLAG_NONE);
-
-	/* NOTE: APPEND and MERGE are not really exclusive they can co-exist */
-	brasero_plugin_set_flags (plugin,
-				  BRASERO_MEDIUM_CD|
-				  BRASERO_MEDIUM_WRITABLE|
-				  BRASERO_MEDIUM_REWRITABLE|
-				  BRASERO_MEDIUM_APPENDABLE|
-				  BRASERO_MEDIUM_HAS_AUDIO|
-				  BRASERO_MEDIUM_HAS_DATA,
-				  BRASERO_BURN_FLAG_APPEND|
-				  BRASERO_BURN_FLAG_MERGE|
-				  BRASERO_BURN_FLAG_BURNPROOF|
-				  BRASERO_BURN_FLAG_OVERBURN|
-				  BRASERO_BURN_FLAG_MULTI|
-				  BRASERO_BURN_FLAG_DUMMY|
 				  BRASERO_BURN_FLAG_NOGRACE,
 				  BRASERO_BURN_FLAG_NONE);
 
@@ -1177,10 +1298,7 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 	 * NOTE: restricted overwrite DVD-RW can't be formatted.
 	 * moreover DVD+RW are formatted while DVD-RW sequential are blanked.
 	 * NOTE: blanking DVD-RW doesn't work */
-	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVD|
-					BRASERO_MEDIUM_PLUS|
-					BRASERO_MEDIUM_REWRITABLE|
-					BRASERO_MEDIUM_APPENDABLE|
+	output = brasero_caps_disc_new (BRASERO_MEDIUM_DVDRW_PLUS|
 					BRASERO_MEDIUM_CLOSED|
 					BRASERO_MEDIUM_HAS_DATA|
 					BRASERO_MEDIUM_UNFORMATTED|
@@ -1189,14 +1307,16 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 	g_slist_free (output);
 
 	/* again DVD+RW don't support dummy */
+	/* NOTE: wodim doesn't support formating already formated DVD+RWs. That
+	 * is an error for it (stupid!). So sets only unformated DVDs. */
 	brasero_plugin_set_blank_flags (plugin,
 					BRASERO_MEDIUM_DVDRW_PLUS|
-					BRASERO_MEDIUM_APPENDABLE|
+					BRASERO_MEDIUM_CLOSED|
 					BRASERO_MEDIUM_HAS_DATA|
 					BRASERO_MEDIUM_UNFORMATTED|
-					BRASERO_MEDIUM_BLANK|
-					BRASERO_MEDIUM_CLOSED,
-					BRASERO_BURN_FLAG_NOGRACE,
+					BRASERO_MEDIUM_BLANK,
+					BRASERO_BURN_FLAG_NOGRACE|
+					BRASERO_BURN_FLAG_FAST_BLANK,
 					BRASERO_BURN_FLAG_NONE);
 
 	/* for blanking (CDRWs) */
@@ -1212,10 +1332,10 @@ brasero_wodim_export_caps (BraseroPlugin *plugin, gchar **error)
 
 	/* add some configure options */
 	immed = brasero_plugin_conf_option_new (GCONF_KEY_IMMEDIATE_FLAG,
-						_("enable -immed flag (see wodim manual)"),
+						_("Enable \"-immed\" flag (see wodim manual)"),
 						BRASERO_PLUGIN_OPTION_BOOL);
 	minbuf = brasero_plugin_conf_option_new (GCONF_KEY_MINBUF_VALUE,
-						 _("minimum drive buffer fill ratio (in %) (see wodim manual):"),
+						 _("Minimum drive buffer fill ratio (in %) (see wodim manual):"),
 						 BRASERO_PLUGIN_OPTION_INT);
 	brasero_plugin_conf_option_int_set_range (minbuf, 25, 95);
 
